unit elementsunit;

(*

This unit was written by Nicola {MSX} Lugato.

http://msx.rules.it
nicola@lugato.net
nicola.lugato@gmail.com

It is released under the term of
The GNU General Public License as published by the Free
Software Foundation.

Description:

This unit contains classes for game entities.

*)


interface

uses Classes,SysUtils, math, meshunit,mediamanagerunit, textureunit, types3d, msxmath, soundunit;

type
TElementList = class;

TElement = class
   list:TElementList;
   dead:boolean;
	 mesh:TMesh;
	 texture:TTexture;
	 orientation:TQuaternion;
	 position:TVector;
	 lights:boolean;
	 constructor Create(aList:TElementList);overload;virtual;
	 constructor Create(aList:TElementList;amesh:TMesh; aTexture:TTexture; aPosition:TVector; aOrientation:TQuaternion);overload;virtual;
	 destructor Destroy; override;
	 procedure Update(delta:single);virtual;
	 procedure Init;virtual;
	 function TranslateToLocalCoords(const v:TVector):TVector;
end;

TShip = class(TElement)
  exploded:boolean;
  speed, maxspeed:tfloat;
  energy, shield:tfloat;
  shots:integer;
  vertSpeed, horizSpeed:single;  // *rotation* speed in vertical and horizontal
  forw, rotateLeft, rotateRight, rotateUp, rotateDown:boolean;
  part:array[0..20] of record pos:TVector;time:single; end;
  sourceLaser:TSource; // sound source for shots
  sourceBoom:TSource; // sound source for boom
  procedure explode;
  procedure Move(delta:single);
  procedure shot;
  function CollisionRouteWith(ship:TShip):boolean;
  function TargetVectorForPursuit(delta:single; TargetShip:TShip; dist:TFloat):TVector;
  function TargetVectorForFlee(delta:single; TargetShip:TShip; dist:TFloat):TVector;
  function TargetVectorForEvade(delta:single; TargetShip:TShip; dist:TFloat):TVector;
  function NeedEnergy:boolean;
  function NeedShots:boolean;
  function NeedShield:boolean;
  destructor Destroy; override;
end;

TRing = class(TElement)
end;

TDebris = class(TElement)
 life:TFloat;
 procedure Init;override;
  procedure Update(delta:single);override;
end;

TPlayer = class(TShip)
  procedure Init;override;
  procedure Update(delta:single);override;
end;

TEnemy = class(TShip) 
  target:TVector;
  targetship:Tship;
  targetring:TRing;
  status:(sEvade, sPursuit, sFlee, sNearingRing, sEnteringRing);
  shottime:single;
  procedure Init;override;
  procedure ChooseTarget(delta:single);
  procedure Update(delta:single);override;
end;

TLaser = class(TElement)
  life:single;
  procedure Update(delta:single);override;
  procedure Init;override;
end;

TExplosion = class(TElement)
  life:single;
  procedure Update(delta:single);override;
  procedure Init;override;
end;

TDrone = class(TElement)
  target:TVector;
  procedure Update(delta:single);override;
  procedure Init;override;
end;



TElementList = class
private
 man:TGenericMediaManager;
 flist:TList;
 function getElement(index:integer):TElement;
 function sort(a,b:TElement):boolean;
public
 (* it's better if we move this to private and add properties *)
 RingShield, RingEnergy, RingShots:TRing;
 Enemy:TEnemy;
 Player:TPlayer;
 source:TSource; // sound source for explosions

 procedure Init;
 function Count:integer;
 function CollideFast(a,b:TElement):boolean;
 procedure Add(e:TElement);
 procedure Update(delta:single);
 procedure Remove(e:TElement);
 constructor Create(manager:TGenericMediaManager);
 property Elements[index:integer]:TElement read getElement; default;
end;

TParticle = record
	pos:TVector;
	life:TFloat;
end;

TDust = array[0..10] of TParticle;


implementation



constructor TElement.Create(aList:TElementList);
begin
        list:=alist;
	identity(orientation);
	position.x:=0;
	position.y:=0;
	position.z:=0;
	dead:=false;
	lights:=true;
	init;
end;

constructor TElement.Create(aList:TElementList; amesh:TMesh; aTexture:TTexture; aPosition:TVector; aOrientation:TQuaternion);
begin
 list:=Alist;
 orientation:=aorientation;
 position:=aposition;
 mesh:=amesh;
 texture:=atexture;
 dead:=false;
 lights:=true;
 init;
end;

procedure TElement.Init;
begin
end;

procedure TElement.Update(delta:single);
var r:TQuaternion;
begin
    SetAxisAngle(r,delta/10,0,0,1);
    MultP(orientation,r);
end;

procedure TLaser.Update(delta:single);
var e:TElement;
begin
	position:=AddRotated(position, orientation, 150*delta);
	life:=life+delta;
	if life>5 then
    begin
      dead:=true;
    end;
end;

procedure TDebris.Update(delta:single);
var e:TElement;
begin
	position:=AddRotated(position, orientation, 50*delta);
	life:=life+delta;
	if life>5 then
    begin
      dead:=true;
    end;
end;

procedure TLaser.Init;
begin
	life:=0;
	lights:=false;
end;

procedure TDebris.Init;
begin
 life:=0;
 orientation:=getRandomQuaternion;
end;

procedure TExplosion.Update(delta:single);
begin
	life:=life+delta;
	if life>0.5 then dead:=true;
end;

procedure TExplosion.Init;
begin
	life:=0;
	lights:=false;
end;


procedure TDrone.Update(delta:single);
begin
  if Distance(position.x, position.y, position.z, target.x, target.y, target.z) < 1 then
    begin
      target.x:=random(100)-50;
      target.y:=random(100);
      target.z:=random(100)-50;
    end;
 position.x:=Interpolate(position.x, target.x, 1*delta);
 position.y:=Interpolate(position.y, target.y, 1*delta);
 position.z:=Interpolate(position.z, target.z, 1*delta);

end;
procedure TShip.Move(delta:single);
var v:TVector;
    r:TQuaternion;
    i:integer;
begin
  for i:=0 to high(part) do
   begin
     if part[i].time<=0 then
      begin
       part[i].timE:=0.1;
       part[i].pos:=AddRotated(position, orientation, getvector(random(100)/200-0.25,random(100)/200-0.25,-3));
      end
     else
      part[i].timE:=part[i].timE-delta;
   end;


  if forw then
    begin
      if speed < maxspeed then speed:=speed+15*delta;
    end
  else
    begin
      if speed > 0 then speed:=speed-15*delta;
    end;

  if energy<=0 then if speed>10 then speed:=10;


	if rotateright then
	 begin	 
	  horizSpeed:=horizSpeed-0.1;
	  if horizSpeed<-1 then horizspeed:=-1;
   end
  else  
		if rotateleft then
		 begin	 
		  horizSpeed:=horizSpeed+0.1;
		  if horizSpeed>1 then horizspeed:=1;
	   end
	  else
	    horizSpeed:=horizSpeed*0.8;  //-sign(horizSpeed)*min(0.1, abs(horizSpeed)) ;
	   
   

	if rotateup then
	 begin	 
	  vertSpeed:=vertSpeed+0.1;
	  if vertSpeed>1 then vertspeed:=1;
   end
  else
		if rotatedown then
		 begin	 
		  vertSpeed:=vertSpeed-0.1;
		  if vertSpeed<-1 then vertspeed:=-1;
		  
	   end
	  else
	   vertSpeed:=vertSpeed*0.8;
   
   
    SetAxisAngle(r,vertSpeed*delta,1,0,0);
    multP(orientation,r);
    
    SetAxisAngle(r,horizSpeed*delta,0,1,0);
    multP(orientation,r);



  position:=AddRotated(position, orientation, speed*delta);
  energy:=energy-speed/20000;  // do this need to be multiplied by delta ?
  if energy < 0 then energy := 0;

   // adjust ship roll
   v:=getvector(0,1,0);                            // the up vector. We well translate as seen by the ship
   v:=Rotated(v, Inverted(orientation));          // translating by the inverse of the rotation
   if abs(v.x)>0.01 then                             // now, if the vector has an X component big enought
     begin
       SetEuler(r, 0, 0, -sign(v.x)*delta);    // we rotate it a bit along his "forward" axis
       multP(orientation,r);
     end;
 sourceLaser.position := position;
 sourceBoom.position := position;
end;

function TShip.CollisionRouteWith(ship:TShip):boolean;
begin
// this will predict ship position
(*
        // distance/speed gives the time that the laser need to reach the target
        t:=dist/(speed*delta);
        // space = time * speed (of target), this gives how much space the ship will cover in the time calculated before
        a:= t * targetship.speed * delta;
        // translate the target position by the distance calculated to get predicted position
        v:=MovePosition(targetship.position, targetship.orientation,a); // position prevision
  *)
end;

function TShip.TargetVectorForPursuit(delta:single; TargetShip:TShip; dist:TFloat):TVector;
var  t, a:TFloat;
begin
  // this function will return a vector for "pursuiting" a ship
  // that is, it will calculate the future ship position so that if you shot, the shot will
  // reach it.
        
  // distance/speed (of the laser) gives the time that the laser need to reach the target
  t:=dist/(150*delta);							
  // space = time * speed (of target), this gives how much space the ship will cover in the time calculated before
  a:= t * TargetShip.speed * delta;		
  // translate the target position by the distance calculated before to get predicted position
  result:=AddRotated(TargetShip.position, TargetShip.orientation,a); // position predicted
  // head to predicted position
  result:=Normalized(TranslateToLocalCoords(result));
end;

function TShip.TargetVectorForEvade(delta:single; TargetShip:TShip; dist:TFloat):TVector;
var  t, a:TFloat;
begin
  // this function will return a vector for "evading" from ship
  // that is, it will try to stay away from the "front" of the ship, so that it should be difficoult
  // for it to aim.

	// THIS IMPLEMENTATIONS IS WRONG
        // this will predict ship position
        
        // distance/speed gives the time that the laser need to reach the target
        t:=dist/(50*delta);							
        // space = time * speed (of target), this gives how much space the ship will cover in the time calculated before
        a:= t * targetship.speed * delta;		
        // translate the target position by the distance calculated to get predicted position
        result:=AddRotated(targetship.position, targetship.orientation,a); // position prevision
        // head to predicted position
        result:=TranslateToLocalCoords(result);

        // invert the vector to evade instead of purge
        result:=Inverted(result);
        
        result:=Normalized(result);
        
        
end;


function TShip.TargetVectorForFlee(delta:single; TargetShip:TShip; dist:TFloat):TVector;
var  t, a:TFloat;
begin
  // this function will return a vector for flee from ship
  // that is, the vector that points away from the target
        
  result:=TranslateToLocalCoords(targetship.position);
  // invert the vector to flee instead of seek
  result:=inverted(result);
end;

procedure TShip.shot;
var e:TElement;
begin
	      if (shots>0) then
	       begin
           // it's laser time!
           e:=TLaser.create(list, list.man.getMeshByName('laser'),list.man.getTextureByName('laser'), addRotated(position,orientation,10), orientation);
		       list.add(e);
           dec(shots);
           sourceLaser.rewind;
           sourceLaser.play;
         end;
end;

procedure TEnemy.ChooseTarget(delta:single);
var v:TVector;
		e:TElement;
		dist,a, t,ringdist:TFloat;
begin
	if shottime>0 then shottime:=shottime-delta*100;
	// distance with the target ship (calculated once, needed a lot)
	dist:=distance(targetship.position, position);
	
	if dist<20 then
	 begin
	 	 // control if we are going to collide with target
	 	 status:=sFlee;
	 end
  else
  if status <> sEnteringRing then
   begin
    if energy < 20 then
    begin
     targetring := list.RingEnergy;
     status:=sNearingRing;
    end
    else if shield <10 then
    begin
     targetring := list.RingShield;
     status:=sNearingRing;
    end
    else if shots < 1 then
    begin
     targetring := list.RingShots;
     status:=sNearingRing;
    end;
  end;
	 
  case status of
    sEvade:
      begin
        target:=TargetVectorForEvade(delta, targetship, dist);
        if dist>200 then status:=sPursuit;
      end;
    sFlee:
      begin
				target:=TargetVectorForFlee(delta, targetship, dist);
        if dist>100 then status:=sEvade;
      end;
    sPursuit:
      begin
        target:=TargetVectorForPursuit(delta, targetship, dist);
        
        if (target.x+target.y)<0.1 then
	      if (shottime<=0) and (shots>0) then
	       begin
           // it's laser time!
           shot;
		       shottime:=20;
         end;
         if dist<50 then status:=sFlee;
      end;
    sNearingRing:
      begin
      	ringdist:=Distance(position, targetring.position); // distance from ring
      	
        v:=Normalized(translatetolocalcoords(targetring.position));
        if ringdist < 200 then
          // if we are near enought, stop evading and go straigt into the ring entrance
          target := v
        else
          // if we are still far from the ring, try evading while nearing the ring
          target:=Interpolated(v,TargetVectorForEvade(delta, targetship, dist), 0.1);
        if ringdist<10 then status:=sEnteringRing;
      end;
    sEnteringRing:
      begin
       target:=getVector(0,1,0);
       if (targetring = list.RingShots) and (shots>0) then status:=sEvade;
       if (targetring = list.RingEnergy) and (energy>10) then status:=sEvade;
       if (targetring = list.RingShield) and (shield>10) then status:=sEvade;
      end;
  end;


 // target is a unit vector that points to the desired location.
 // it is relative to this object position and orientation (local coordinates)
 rotateup:=(target.y<-0.01);
 rotatedown:=(target.y>0.01);
 rotateleft:=(target.x>0.01);
 rotateright:=(target.x<-0.01);
 // avoid not turning if perfectly aligned (could be replaced with a nice dot product)
 if target.z<-0.9 then
  begin
   rotatedown:=true;
   rotateleft:=true;
   // writeln('corrected '+floattostr(target.z));
  end;
  
 forw:=true;

end;



procedure TPlayer.Update(delta:single);
begin
 if exploded then exit;
 move(delta);
 if (shield<=0) and not exploded then
  begin
    explode;
  end;
end;

procedure TShip.explode;
var v:TVector;
    r:TQuaternion;
    e:TElement;
    i:integer;
    
begin
 for i:=0 to 20 do
   begin
    e:=TDebris.create(list, list.man.getMeshByName('debris'),list.man.getTextureByName('ship2'), position, orientation);
    list.add(e);
   end;
 e:=TExplosion.create(list, nil ,nil, position, orientation);
 list.add(e);
 list.source.position:=position;
 list.source.rewind;
 list.source.play;
 exploded:=true;
end;

procedure TEnemy.Update(delta:single);
begin
 if exploded then exit;
 choosetarget(delta);
 move(delta);
 if (shield<=0) and not exploded then
  begin
    explode;
  end;
end;



procedure TDrone.Init;
begin
 // this will force it to choos new target next step
 target:=position;
end;


procedure TPlayer.Init;
var i:integer;
begin
  exploded:=falsE;
  vertSpeed:=0;
  horizSpeed:=0;
  rotateLeft:=false;
  rotateRight:=false;
  rotateUp:=false;
  rotateDown:=false;
  forw:=falsE;
  maxspeed:=50;
  energy:=100;
  shield:=100;
  shots:=100;
  sourceLaser:=TSource.create(list.man.getSoundByName('laser'));
  sourceBoom:=TSource.create(list.man.getSoundByName('boom'));  
  for i:=0 to high(part) do
   begin
     part[i].time:=0;
   end;
end;

procedure TEnemy.Init;
begin
  exploded:=falsE;
  vertSpeed:=0;
  horizSpeed:=0;
  maxspeed:=40;
  rotateLeft:=false;
  rotateRight:=false;
  rotateUp:=false;
  rotateDown:=false;
  forw:=falsE;
  energy:=100;
  shield:=100;
  shots:=100;
  status:=sPursuit;
  sourceLaser:=TSource.create(list.man.getSoundByName('laser'));
  sourceBoom:=TSource.create(list.man.getSoundByName('boom'));
  shottime:=0;
end;

function TElement.TranslateToLocalCoords(const v:TVector):TVector;
begin
 result:=TranslateToFrame(v, position, orientation);
end;

function TElementList.CollideFast(a,b:TElement):boolean;
var d:TFloat;
begin
d:=distance(a.position, b.position);
result := (a.mesh.getBoundingSphereRadius + b.mesh.getBoundingSphereRadius) > d;
end;

constructor TElementList.Create(manager:TGenericMediaManager);
begin
	man:=manager;
	flist:=TList.create;
	source:=TSource.create(man.getSoundByName('large'));
end;

procedure TElementList.Add(e:TElement);
begin
	flist.add(e);
end;

function TElementList.getElement(index:integer):TElement;
begin
	result:=TElement(flist[index]);
end;

function TElementList.Count:integer;
begin
	result:=flist.count;
end;

procedure TElementList.Remove(e:TElement);
begin
	flist.remove(e);
end;

procedure TElementList.Update(delta:single);
var i:integer;
		p:TElement;
begin
	for i:=0 to count-1 do
	 begin
     elements[i].update(delta);
   end;
	
  for i:=count-1 downto 0 do
   if elements[i].dead then
    begin
    	p:=elements[i];
    	remove(p);
    	p.free;
    end;
end;

function TElementList.sort(a,b:TElement):boolean;
begin
	result:=a.position.x > b.position.x;
end;

function TShip.NeedEnergy:boolean;
begin
	 result:=False;
end;
function TShip.NeedShots:boolean;
begin
	if shield=0 then result:=true else result:=False;
end;
function TShip.NeedShield:boolean;
begin
	if shield<10 then result:=true else result:=False;
end;

procedure TElementList.Init;
var i:integer;
    p:TElement;
begin
while count>0 do
 begin
  p:=Elements[0];
  Remove(p);
  p.free;
 end;
 
Player:=TPlayer.create(self);
Player.mesh:=man.getMeshByName('ship');
Player.texture:=man.getTextureByName('ship2');
Player.position:=getVector(0.1,0.1,0.1);
add(Player);

ringEnergy:=TRing.create(self);
ringEnergy.mesh:=man.getMeshByName('ring');
ringEnergy.texture:=man.getTextureByName('ring');
ringEnergy.position:=getVector(-300,-120,500);
add(ringEnergy);


ringShots:=TRing.create(self);
ringShots.mesh:=man.getMeshByName('ring');
ringShots.texture:=man.getTextureByName('ring');
ringShots.position:=getVector(0,300,-200);
add(ringShots);

ringShield:=TRing.create(self);
ringShield.mesh:=man.getMeshByName('ring');
ringShield.texture:=man.getTextureByName('ring');
ringShield.position:=getVector(800,-500,280);
add(ringShield);



enemy:=TEnemy.create(self);
enemy.mesh:=man.getMeshByName('ship');
enemy.texture:=man.getTextureByName('ship2');
enemy.position:=getVector(70,0,300);
enemy.orientation:=SetAxisAngle(90, 0,0,1);
enemy.targetship:=player;
add(enemy);

end;

destructor TElement.Destroy;
begin
	inherited Destroy;
end;

destructor TShip.Destroy;
begin
	sourceLaser.free;
	sourceBoom.free;
	inherited Destroy;
end;

end.
