unit mainunit;

(*

This unit was written by Nicola {MSX} Lugato.

http://msx.rules.it
nicola@lugato.net
nicola.lugato@gmail.com

It is released under the term of
The GNU General Public License as published by the Free
Software Foundation.

This game is dedicated to Fabiana

Description:

This is the main file of the game.
The TProg class represent the program.

*)

interface


uses
  openglmainloop, screenunit, openglunit, meshunit,
  SDL, GLU, GL, mediamanagerunit,elementsunit,
  types3d, SysUtils, stringdrawer2, glmeshunit, soundunit,
  frustumunit, math, msxmath, textureunit,
  keyboardunit;

const CAM_DISTANCE:single = 25;									// distance between the camera and the ship
      CAM_ABOVE:single = 5;									    // how much the camera is above the ship
      CAM_DELAY:single = 3;						  // speed of camera in following the ship

const GL_CLAMP_TO_EDGE = $812F; // missing on gl.pas!?

const WelcomeMsg = 
'                                                                                                              '+
'- Space Cadett -    by Nicola {MSX} Lugato   -   This game is my entry for the PGD Dogfight Game Programmming Competition   15/03/2005      '+
'Keys:      CTRL: laser   SHIFT: engines    ESC: quit       '+
'Drive the Space Cadett against his enemy!      Use the Ring Rechargers to refill energy, shields and lasers!      '+
'This game is dedicated to Fabiana                   Press CTRL to start!';

// this class simply represent the program
Type TProg = class
  frustumculler:TFrustumCuller;                 // frustum culler
  screen:tscreen;                               // the opengl screen
  Loop:TOpenGlRealTimeLoop;                     // the opengl game loop
  man:TGenericMediaManager;                     // the media manager

  camquat:TQuaternion;                          // camera quaternion

  Keystate:TkeyState;                           // the keyboard state

  game:TElementList;                            // the game data
  
  lasttick, lastframetime: cardinal;            // frame timing
  
  introtime:tfloat;                             // introduction timing
  
  gamestatus:(gsIntro, gsGame, gsOutro);                 // the state of the program
  
  Dust:TDust;                                   // the dust particle that surround the player
  
  MiniCup:TTexture;																// little cup texture
  PlayerCups:integer;
  EnemyCups:integer;

  winner:boolean;                               // win last match

  procedure DoLoop;
  procedure DrawStuff;
  procedure Update(delta:single);
  procedure init;

  procedure Start;
  procedure drawSpot(const pos:TVector; tilesize:single; r,g,b:TFloat);
  procedure DrawGrid;
  procedure shot;
  procedure DrawIndicators;
  procedure DrawSkyBox;
  procedure ApplyQuat(const q:TQuaternion);
  procedure ApplyQuatInverse(const q:TQuaternion);
  //procedure ApplyQuatInverseOnlyY(const q:TQuaternion);
  procedure Cleanup;
  //procedure drawMap;
  Procedure UpdateIntro(delta:TFloat);
  procedure DrawIntro;
  Procedure UpdateOutro(delta:TFloat);
  procedure DrawOutro;
  procedure DrawCrossHair;
  procedure DrawTargetArrow(v:TVector;base:TQuaternion;x,y,r,g,b:tfloat);
  procedure HandleParticle(delta:TFloat);
  procedure beginOrtho;
  procedure endOrtho;
  procedure DrawCompass;
  procedure KeyDownEvent(key:TSDL_KeyboardEvent);
  procedure DrawElement(e:TElement);
  procedure KeyUpEvent(key:TSDL_KeyboardEvent);
  procedure OrientRing(ring, player, enemy:TElement);
end;



implementation

procedure TProg.drawSpot(const pos:TVector; tilesize:single; r,g,b:TFloat);
begin
glPushMatrix;
glLoadIdentity;

  // this is deep shit!
  gltranslatef(0,-CAM_ABOVE,-CAM_DISTANCE);

  glRotatef(180, 0,1,0);
  applyquatinverse(camquat);

  with game.player.position do
       gltranslatef(-x,-y,-z);

glTranslatef(pos.x,pos.y, pos.z);

  applyquat(camquat);
  

glDisable(GL_CULL_FACE);
glEnable(GL_BLEND);
//glBlendFunc(GL_ONE,GL_ONE_MINUS_SRC_COLOR );
glBlendFunc(GL_SRC_ALPHA,GL_ONE);
glColor3f(r,g,b);
man.getTextureByName('spot').bind;

     glBegin(GL_TRIANGLE_strip);

     glTexCoord2f(0,0);
     glVertex2f(-TileSize, -TileSize);

     glTexCoord2f(0,1);
     glVertex2f(-TileSize,  TileSize);


     glTexCoord2f(1,0);
     glVertex2f(TileSize,  -TileSize);

     glTexCoord2f(1,1);
     glVertex2f(TileSize,  TileSize);

     glEnd;

glEnable(GL_CULL_FACE);
glDisable(GL_BLEND);
glPopMatrix;
end;

procedure TProg.HandleParticle(delta:TFloat);
var i:integer;
begin
	for i:=0 to high(dust) do
	 with dust[i] do
		 begin
		 	 if (abs(pos.x-game.player.position.x) + abs(pos.y-game.player.position.y) + abs(pos.z-game.player.position.z)) > 50 then
		 	  begin
		 	  	pos:=AddRotated(game.player.position,game.player.orientation,30);
		 	  	pos:=add (pos, getVector(random(100)/5-10,random(100)/5-10,random(100)/5-5));
		 	  end;
		 end;
end;

procedure TProg.OrientRing(ring, player, enemy:TElement);
var e:TElement;
    r:TQuaternion;
begin
    if distance(Ring.position, player.position) < distance(Ring.position, enemy.position) then
     e:=player else e:=enemy;
    r:=LookAt(sub(e.position, Ring.position),getVector(0,1,0));
    Ring.orientation:=Slerp(Ring.orientation, r, 0.02);

end;

procedure TProg.Update(delta:single);
var r:TQuaternion;
    up,dir:TVector;
    p:pointer;
    i:integer;
    e,x:TElement;
begin

 if game.player.exploded or game.enemy.exploded then
   begin
     introtime:=introtime+delta;
     if introtime> 5 then
       begin
        introtime:=0;
        winner:=not game.player.exploded;
        if winner then inc(playercups) else inc(enemycups);
        gamestatus:=gsOutro;
       end;
   end;


 handleparticle(delta);

 with game do
   begin

    OrientRing(ringShots, player, enemy);
    OrientRing(ringShield, player, enemy);
    OrientRing(ringEnergy, player, enemy);

   end;
 game.update(delta);
 
 
 
 //
 // MOST OF THE FOLLOWING MUST BE MOVED ON GAME.UPDATE
 
 For i:=0 to game.count-1 do
  begin
   e:=game[i];
   if (e is TLaser) then
    begin
       if not game.enemy.exploded then
    	 if (game.CollideFast(e, game.enemy)) then
     	 	begin
    	 		game.enemy.shield:=game.enemy.shield-2;

          game.enemy.sourceBoom.stop;
          game.enemy.sourceBoom.rewind;
          game.enemy.sourceBoom.play;

    	 		e.dead:=true;
          x:=TExplosion.create(game, nil ,nil, e.position, e.orientation);
          game.add(x);
    	 	end;


       if not game.player.exploded then
    	 if (game.CollideFast(e, game.player)) then
    	 	begin
    	 		game.player.shield:=game.player.shield-2;

          game.Player.sourceBoom.stop;
          game.Player.sourceBoom.rewind;
          game.Player.sourceBoom.play;

    	 		e.dead:=true;
          x:=TExplosion.create(game, nil ,nil, e.position, e.orientation);
          game.add(x);
    	 	end;
    end;
  end;
	
	with game do
	 begin
	  if game.CollideFast(ringenergy, player) then player.energy:=100;
	  if game.CollideFast(ringshots, player) then player.shots:=100;
	  if game.CollideFast(ringshield, player) then
      begin
        player.shield:=100;
        player.shots:=0;
      end;

	 
	  if game.CollideFast(ringenergy, enemy) then enemy.energy:=100;
	  if game.CollideFast(ringshots, enemy) then enemy.shots:=100;
	  if game.CollideFast(ringshield, enemy) then
      begin
        enemy.shield:=100;
        enemy.shots:=0;
      end;
   end;
	
	// this is where the camera orientation is interpolate to follow player's ship orientation
  camquat:=Slerp(camquat, game.player.orientation, CAM_DELAY*delta);
 
 // set the player ship controls as the keyboard input
 with game.player do
  begin
  forw:=keystate.shift;
	rotateLeft:=keystate.left;
	rotateRight:=keystate.right;
	rotateUp:=keystate.up;
	rotateDown:=keystate.Down;
 end;

 setListenerPosition(game.player.position);
 getDirUpVectors(game.player.orientation, dir, up);
 setListenerOrientation(dir, up);
end;

procedure TProg.init;
var i:integer;
    e:TElement;
    source:TSource;
begin
	
for i:=0 to high(dust) do
	 dust[i].life:=0;
	
identity(camquat);
game:=TElementList.create(man);

game.init;
playercups:=0;
enemycups:=0;
minicup:=man.getTextureByName('minicup');	
(*
source:=TSource.create(man.getSoundByName('engine-1'));
source.Loop:=true;
source.Position:=getVector(0,0,0);
source.Play;
*)
end;

procedure TProg.ApplyQuat(const q:TQuaternion);
var x,y,z,a:TFloat;
begin
	GetAxisAngle(q, x,y,z,a);
	glRotatef(a*180/pi, x, y, z);
end;

procedure TProg.ApplyQuatInverse(const q:TQuaternion);
var x,y,z,a:single;
begin
	GetAxisAngle(q, x,y,z,a);
	glRotatef(a*180/pi, -x, -y, -z);
end;

(*procedure TProg.ApplyQuatInverseOnlyY(const q:TQuaternion);
var x,y,z,a:single;
begin

	GetAxisAngle(q, x,y,z,a);
	glRotatef(a*180/pi, 0, -y, 0);
end;*)

procedure TProg.DrawElement(e:TElement);
begin
  glColor3f(1,1,1);
  glPushMatrix;          
  // set up the tranformation
  with e.position do
  	glTranslatef(x,y,z);
	applyquat(e.orientation);	

  e.texture.bind;        
  e.mesh.draw;

  glPopMatrix;
end;

procedure TProg.DoLoop;
var i:cardinal;
		s:single;
begin
  // calculate the duration of the last frame, storing it in lastframetime
  i:=SDL_GetTicks;
  lastframetime:=i-lasttick;
  lasttick:=i;
  // update the game according to the duration of the frame
  s:=max(epsilon, lastframetime/1000);  // avoid div by zero
  case gamestatus of
   gsIntro:
     begin
       UpdateIntro(s);
       if gamestatus=gsIntro then // don't draw is status changed
        DrawIntro;
     end;
   gsOutro:
     begin
       UpdateOutro(s);
       if gamestatus=gsOutro then // don't draw is status changed
         DrawOutro;
     end;
   gsGame:
     begin
       Update(s);
       if gamestatus=gsGame then // don't draw is status changed
         DrawStuff;
     end;
   end;
end;

Procedure TProg.UpdateIntro(delta:TFloat);
begin
 introtime:=introtime+delta;
 if Keystate.ctrl then
  begin
   gamestatus:=gsgame;
//   Keystate.ctrl:=false; //avoid shoting when change state
   introtime:=0;
  end;
end;

Procedure TProg.UpdateOutro(delta:TFloat);
begin
 introtime:=introtime+delta;
 if Keystate.ctrl then
  begin
   gamestatus:=gsgame;
   game.init;
   identity(camquat);
   introtime:=0;
  end;
end;

procedure TProg.DrawIntro;
var v:TFloat;
begin
  if (lastframetime<=0) or (lastframetime>100) then exit;


  glEnable(GL_TEXTURE_2D);
  
  glMatrixMode(GL_MODELVIEW);   // Change Projection to Matrix Mode
  

  // let's clear the frame to the right color and the depth buffer
  glClearColor(0,0,0,1);
  glClear( GL_COLOR_BUFFER_BIT or GL_DEPTH_BUFFER_BIT );

  beginOrtho;
  glLoadIdentity;
  man.getTextureByName('title').bind;


  glBegin( GL_TRIANGLE_STRIP  );
    glColor3f(1,1,1);
    glTexCoord2f(0,1);
    glVertex3f(  0,  0, -100 );

    glTexCoord2f(1,1);
    glVertex3f(  100,  0, -100 );

    glTexCoord2f(0,0);
    glVertex3f(  0,  100, -100 );

    glTexCoord2f(1,0);
    glVertex3f(  100,  100, -100 );
  glEnd;

  endOrtho;
  
  // start calculation of the matrix (by loading identity)
  glLoadIdentity;
  gluLookAt(0,15,-20, 0, 0,0,0,1,0);

  glRotatef(introtime*20, 0, 1,0);


  man.getTextureByName('ship2').bind;
  man.getMeshByName('ship').draw;

  v:=introtime *5;
beginDrawString;

  man.getTextureByName('font').bind;
  drawString(20-((trunc(v*8)) mod 8),555,copy(WelcomeMsg,trunc(v), 100),1,1,1);
  
  
endDrawString;  


  // finally, show our nice frame!
  SDL_GL_SwapBuffers;
  sleep(10); // this avoids some problems if the rendering is too fast

end;


procedure TProg.DrawOutro;
begin
  if (lastframetime<=0) or (lastframetime>100) then exit;
  // this is the drawing pipeline

  //glDisable(GL_DEPTH_TEST);
  glEnable(GL_TEXTURE_2D);

  glMatrixMode(GL_MODELVIEW);   // Change Projection to Matrix Mode


  // let's clear the frame to the right color and the depth buffer
  glClearColor(0,0,0,1);
  glClear( GL_COLOR_BUFFER_BIT or GL_DEPTH_BUFFER_BIT );

  beginOrtho;
  glLoadIdentity;
  if winner then
   man.getTextureByName('youwin').bind
  else
   man.getTextureByName('youlose').bind;
  


  glBegin( GL_TRIANGLE_STRIP  );         // Drawing Using Triangles
    glColor3f(1,1,1);
    glTexCoord2f(0,1);
    glVertex3f(  0,  0, -100 );

    glTexCoord2f(1,1);
    glVertex3f(  100,  0, -100 );

    glTexCoord2f(0,0);
    glVertex3f(  0,  100, -100 );


    glTexCoord2f(1,0);
    glVertex3f(  100,  100, -100 );
  glEnd;                            // Finished Drawing The Triangle

  endOrtho;

  // start calculation of the matrix (by loading identity)
  glLoadIdentity;
  gluLookAt(0,10,-15, 0,3,0,0,1,0);

  glRotatef(introtime*10, 0, 1,0);

  // man.getTextureByName('ship2').bind;
  glEnable(GL_TEXTURE_2D);

  if winner then
    begin
			man.getTextureByName('cup').bind;
		  man.getMeshByName('cup').draw;
    end
  else
    begin
			man.getTextureByName('cross').bind;
		  man.getMeshByName('cross').draw;
    end;

  // finally, show our nice frame!
  SDL_GL_SwapBuffers;
  sleep(10); // this avoids some problems if the rendering is too fast

end;



procedure TProg.DrawStuff;
var i:integer;
    c:cardinal;
    v:TVector;
    r:TFloat;
begin
  if (lastframetime<=0) or (lastframetime>100) then exit;
  // this is the drawing pipeline
  
  glMatrixMode(GL_MODELVIEW);   // Change Projection to Matrix Mode

  // let's clear the frame to the right color and the depth buffer
  glClearColor(0,0,0,1);
  glClear( GL_COLOR_BUFFER_BIT or GL_DEPTH_BUFFER_BIT );


  // start calculation of the matrix (by loading identity)
  glLoadIdentity;
  

  gltranslatef(0,-CAM_ABOVE,-CAM_DISTANCE);
  
  glRotatef(180, 0,1,0);
  applyquatinverse(camquat);

  with game.player.position do
       gltranslatef(-x,-y,-z);

  // update the frustum culler to the new view matrix
  frustumculler.Calculate;

  // the sky box
  glPushMatrix;
  glLoadIdentity;
  glRotatef(180, 0,1,0);
  applyquatinverse(camquat);
  
  DrawSkyBox;
  glPopMatrix;
  
//  draworiginlines(10);
  
  for i:=0 to game.count-1 do
    if game[i].mesh <> nil then
      if (not (game[i] is TShip)) or (not TShip(game[i]).exploded) then  // really ugly solution to hide ships after they explode
        drawelement(game[i]);

  

  
	glColor3f(1,1,1);
  glPushMatrix;          
	glBegin(GL_POINTS);
  for i:=0 to high(dust) do
	 glVertex3f(  dust[i].pos.x,dust[i].pos.y,dust[i].pos.z );
	glEnd; 


  glPopMatrix;
  
  
  DrawSpot(game.ringEnergy.position, 15,1,1,0);
  DrawSpot(game.ringShots.position, 15,0,0,1);
  DrawSpot(game.ringShield.position, 15,0,1,0);
  glDepthMask(GL_FALSE);
  
  if not game.player.exploded then
    for i:=0 to high(game.player.part) do
     DrawSpot(game.player.part[i].pos, 0.5,1,0,0);
  if not game.enemy.exploded then
    for i:=0 to high(game.enemy.part) do
     DrawSpot(game.enemy.part[i].pos, 0.5,1,0,0);

  for i:=0 to game.count-1 do
    if game[i].mesh = nil then
      drawSpot(game[i].position,4-TLaser(game[i]).life*8,1,TLaser(game[i]).life,0); // explosion

  
  glDepthMask(GL_TRUE);

  
  glDisable(GL_DEPTH_TEST);

    with game.enemy do
     if not exploded then
      if not frustumculler.IsSphereWithin(position.x, position.Y, position.Z, mesh.getBoundingSphereRadius) then
        DrawTargetArrow(sub(position, game.player.position),game.player.orientation, 50,20,1,0,0);

    with game.ringEnergy do
      if not frustumculler.IsSphereWithin(position.x, position.Y, position.Z, mesh.getBoundingSphereRadius) then
        DrawTargetArrow(sub(position, game.player.position),game.player.orientation, 50,0,1,1,0);

    with game.ringShots do
      if not frustumculler.IsSphereWithin(position.x, position.Y, position.Z, mesh.getBoundingSphereRadius) then
        DrawTargetArrow(sub(position, game.player.position),game.player.orientation, 50,-10,0,0,1);

    with game.ringShield do
      if not frustumculler.IsSphereWithin(position.x, position.Y, position.Z, mesh.getBoundingSphereRadius) then
        DrawTargetArrow(sub(position, game.player.position),game.player.orientation, 50,-20,0,1,0);

  beginOrtho;

    drawIndicators;

    DrawCrossHair;

    DrawCompass;

  endOrtho;
  
  
beginDrawString;

  man.getTextureByName('font').bind;
  drawStringV(20,455,'speed',1,1,1);
  
  drawStringV(35,455,'fuel',1,1,1);
  
  drawStringV(50,455,'shots',1,1,1);
  
  drawStringV(85,455,'player',1,1,1);
  drawStringV(115,455,'enemy',1,1,1);
  
  drawString(10,5,'Player:',1,1,1);
  drawString(10,50,'Enemy:',1,1,1);
  minicup.bind;
  for i:=0 to playercups-1 do
    DrawRectangle(10+i*30,15,24,24,1,1,1);
  for i:=0 to enemycups-1 do
    DrawRectangle(10+i*30,60,24,24,1,1,1);
endDrawString;  
  

  // finally, show our nice frame!
  SDL_GL_SwapBuffers;

  // Sleep(60); // for testing the game as in slow computer :P
end;

procedure TProg.DrawIndicators;
begin
  // first we must switch the projection to orthographic
  // becouse it should not be in perspective like objects of the game

  glEnable(gl_Blend);
  glDisable(GL_DEPTH_TEST);
  glDisable(GL_TEXTURE_2D);
  glDisable(GL_CULL_FACE);

  glLoadIdentity;                          
  DrawRectangle(2,25,2,game.player.speed/2,1,1,1);
  
  DrawRectangle(4,25,2,game.player.energy/4,1,1,0);
  
  DrawRectangle(6,25,2,game.player.shots/4,0,0,1);

  DrawRectangle(10,25,2,game.player.shield/4,0,1,0);
  
  DrawRectangle(14,25,2,game.enemy.shield/4,1,0,0);

  
  glEnable(GL_TEXTURE_2D);

  glEnable(GL_DEPTH_TEST);
  glEnable(GL_CULL_FACE);
end;

procedure TProg.beginOrtho;
begin
  glMatrixMode( GL_PROJECTION );             // tells we are going to work in the projection matrix stack
  glPushMatrix;                              // save the old projection matrix (ie, the perspective one)
  glLoadIdentity;                            // must reset the projection
  glOrtho(0,100,0,100,-100,100);                 // and load a simple orthographic
  glMatrixMode(GL_MODELVIEW);                // now back to the modelview
end;


procedure TProg.endOrtho;
begin
  glMatrixMode( GL_PROJECTION );             // back to projection mode..
  glPopMatrix;                               // to restore the good old perspective
  glMatrixMode(GL_MODELVIEW);
end;

procedure TProg.DrawCompass;
begin
  glEnable(GL_TEXTURE_2D);
  glEnable(gl_Blend);                        // the compass is blended
  
  glLoadIdentity;
  glTranslatef(8,10,0);                     // put it in the lower left corner
  glscalef(5,5,5);
  glColor3f(1,1,1);                          // no color
  glRotatef(180, 0,1,0);                     // damn it!
  applyquatinverse(game.player.orientation);        // apply the inverse matrix, so that the compass is allineated to the ground
  man.getTextureByName('compass').bind;      // choose texture
  man.getMeshByName('compass').draw;         // draw model
  
  glDisable(gl_Blend);                       // no more blend
  glDisable(GL_TEXTURE_2D);
  
end;

procedure TProg.DrawTargetArrow(v:TVector;base:TQuaternion;x,y,r,g,b:tfloat);
var q:TQuaternion;
begin
  glEnable(GL_TEXTURE_2D);
  glDisable(gl_Blend);                        // the compass is blended

  glLoadIdentity;
  glTranslatef(x,y,-100);                     // put it in the lower left corner
  glColor3f(r,g,b);                          // no color

  glRotatef(180, 0,1,0);                     // damn it!
//  applyquatinverse(ship.orientation);        // apply the inverse matrix, so that the compass is allineated to the ground
  applyquatinverse(base);
  q:=LookAt(v, getvector(0,1,0));
  applyquat(q);

  
  man.getTextureByName('arrow').bind;      // choose texture
  man.getMeshByName('arrow').draw;         // draw model

//  glDisable(gl_Blend);                       // no more blend
//  glDisable(GL_TEXTURE_2D);

end;



procedure TProg.DrawCrossHair;
begin
  // draw crosshair
  glDisable(GL_TEXTURE_2D);
  glLoadIdentity;
  glBegin(GL_LINES);
    glColor3f(   1,  0, 0 );

    glVertex2f(  49, 50 );
    glVertex2f(  51, 50 );

    glVertex2f(  50, 49 );
    glVertex2f(  50, 51 );

  glEnd;
end;

procedure TProg.KeyDownEvent(key:TSDL_KeyboardEvent);
begin
case key.keysym.sym of
  SDLK_RIGHT: keystate.right := true;
  SDLK_LEFT: keystate.left := true;
  SDLK_UP: keystate.up:=true;
  SDLK_TAB: 
   begin
   	 keystate.tab:=true;
         if gamestatus=gsGame then shot;
   end;
  SDLK_RCTRL, 
  SDLK_LCTRL: 
   begin
   	 keystate.ctrl:=true;
         if gamestatus=gsGame then shot;
   end;
  SDLK_DOWN: keystate.down:=true;
  SDLK_RSHIFT,
  SDLK_LSHIFT:
    begin
     keystate.shift:=true;
     //man.getSoundByName('horn').Play;
    end;
  SDLK_ESCAPE: loop.StopLoop;
end;
end;

procedure TProg.Shot;
begin
 game.player.shot;
end;

procedure TProg.KeyUpEvent(key:TSDL_KeyboardEvent);
begin
case key.keysym.sym of
  SDLK_RIGHT: keystate.right := false;
  SDLK_UP: keystate.up:=false;
  SDLK_DOWN: keystate.down:=false;
  SDLK_TAB: keystate.tab:=false;
  SDLK_RCTRL, 
  SDLK_LCTRL: keystate.ctrl:=false;  
  SDLK_RSHIFT,
  SDLK_LSHIFT: keystate.shift:=false;
  SDLK_LEFT: keystate.left := false;
end;
end;

procedure TProg.Cleanup;
begin
	screen.finalize;
end;

procedure TProg.Start;
begin
// this method sets up all the game.

 gamestatus:=gsIntro;

 // obtain our screen
 screen:=TScreen.Create('Space Cadett by {MSX}!', 800, 600, 16, ParamStr(1)='-fs', 5000); (* note: error handling need to be completely revised! *)

 if not screen.Initialize then exit;

 if not initsound then exit;

 keystate:=tkeystate.create;

 frustumculler:=TFrustumCuller.create;

 // default: texture active
 glEnable(GL_TEXTURE_2D);

 // glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
 glEnable(GL_CULL_FACE);
 glCullFace(GL_Back);

 // start up our kind media manager
 man:=TFileMediaManager.Create;

 // set up the main loop
 Loop:=TOpenGlRealTimeLoop.create(DoLoop);
 loop.setKeyDownEvent(KeyDownEvent);
 loop.setKeyUpEvent(KeyUpEvent);

 // initialize some other things
 init;

 glDisable(GL_LIGHTING);


(* glFogi(GL_FOG_MODE, GL_LINEAR);		// Fog Mode
 glFogfv(GL_FOG_COLOR, @(FogColor));			// Set Fog Color
 glFogf(GL_FOG_DENSITY, 0.05);				// How Dense Will The Fog Be
 glHint(GL_FOG_HINT, GL_DONT_CARE);			// Fog Hint Value
 glFogf(GL_FOG_START, 250);				// Fog Start Depth
 glFogf(GL_FOG_END,   450);				// Fog End Depth
 glEnable(GL_FOG);					// Enables GL_FOG
*)



 // ready to go! :P
 loop.StartLoop;
end;







(*
Draws a skybox around the scene.
This is not used at the time.
*)
procedure TProg.DrawSkyBox;
var cl:integer;
begin
 cl:=5;

  glEnable(GL_TEXTURE_2D);
  glDisable(GL_DEPTH_TEST);

  glColor3f(1,1,1);
  //man.getTextureByName('cliffFront').bind;
  man.getTextureByName('sky2').bind;
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);

  glBegin(GL_QUADS);
    glTexCoord2f(0.0, 0.0); glVertex3f(-cl, cl,  cl);
    glTexCoord2f(1.0, 0.0); glVertex3f( cl, cl,  cl);
    glTexCoord2f(1.0, 1.0); glVertex3f( cl, -cl,  cl);
    glTexCoord2f(0.0, 1.0); glVertex3f(-cl, -cl,  cl);
  glend;

  //man.getTextureByName('cliffBack').bind;
  man.getTextureByName('sky2').bind;
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);

  glBegin(GL_QUADS);
    glNormal3f( 0.0, 0.0,-1.0);
    glTexCoord2f(1.0, 0.0); glVertex3f(-cl, cl, -cl);
    glTexCoord2f(1.0, 1.0); glVertex3f(-cl, -cl, -cl);
    glTexCoord2f(0.0, 1.0); glVertex3f( cl, -cl, -cl);
    glTexCoord2f(0.0, 0.0); glVertex3f( cl, cl, -cl);
  glEnd;

  //man.getTextureByName('cliffBottom').bind;
  man.getTextureByName('sky').bind;
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);

  glBegin(GL_QUADS);
    glNormal3f( 0.0, 1.0, 0.0);
    glTexCoord2f(0.0, 0.0); glVertex3f(-cl,  -cl, -cl);
    glTexCoord2f(1.0, 0.0); glVertex3f(-cl,  -cl,  cl);
    glTexCoord2f(1.0, 1.0); glVertex3f( cl,  -cl,  cl);
    glTexCoord2f(0.0, 1.0); glVertex3f( cl,  -cl, -cl);
  glEnd;

  //man.getTextureByName('cliffTop').bind;
  man.getTextureByName('sky3').bind;
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);

  glBegin(GL_QUADS);
    glNormal3f( 0.0,-1.0, 0.0);
    glTexCoord2f(0.0, 1.0); glVertex3f(-cl, cl, -cl);
    glTexCoord2f(0.0, 0.0); glVertex3f( cl, cl, -cl);
    glTexCoord2f(1.0, 0.0); glVertex3f( cl, cl,  cl);
    glTexCoord2f(1.0, 1.0); glVertex3f(-cl, cl,  cl);
  glEnd;

  //man.getTextureByName('cliffLeft').bind;
  man.getTextureByName('sky2').bind;
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);

  glBegin(GL_QUADS);
    glNormal3f( 1.0, 0.0, 0.0);
    glTexCoord2f(1.0, 0.0); glVertex3f( cl, cl, -cl);
    glTexCoord2f(1.0, 1.0); glVertex3f( cl, -cl, -cl);
    glTexCoord2f(0.0, 1.0); glVertex3f( cl, -cl,  cl);
    glTexCoord2f(0.0, 0.0); glVertex3f( cl, cl,  cl);
  glEnd;

  //man.getTextureByName('cliffRight').bind;
  man.getTextureByName('sky4').bind;
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);

  glBegin(GL_QUADS);
    glNormal3f(-1.0, 0.0, 0.0);
    glTexCoord2f(0.0, 0.0); glVertex3f(-cl, cl, -cl);
    glTexCoord2f(1.0, 0.0); glVertex3f(-cl, cl,  cl);
    glTexCoord2f(1.0, 1.0); glVertex3f(-cl, -cl,  cl);
    glTexCoord2f(0.0, 1.0); glVertex3f(-cl, -cl, -cl);

  glEnd();

  glEnable(GL_DEPTH_TEST);

end;

procedure TProg.DrawGrid;
var mode:cardinal;
    TileSize:integer;
    si:integer;
begin
TileSize:=4;
si:=64;

mode:=GL_TRIANGLE_FAN;
glEnable(GL_TEXTURE_2D);

man.getTextureByName('marmoblu').bind;

   glBegin(mode);

     glTexCoord2f(0,0);
     glVertex3f(0, 0, 0);

     glTexCoord2f(0,1*si);
     glVertex3f(0, 0, (si)*TileSize);

     glTexCoord2f(1*si,1*si);
     glVertex3f((si)*TileSize, 0, (si)*TileSize);

     glTexCoord2f(1*si,0);
     glVertex3f((si)*TileSize, 0, 0);

     glTexCoord2f(1*si,-1*si);
     glVertex3f((si)*TileSize, 0, -(si)*TileSize);

     glTexCoord2f(0,-1*si);
     glVertex3f(0, 0, -(si)*TileSize);

     glTexCoord2f(-1*si,-1*si);
     glVertex3f(-(si)*TileSize, 0, -(si)*TileSize);

     glTexCoord2f(-1*si,0);
     glVertex3f(-(si)*TileSize, 0, 0);

     glTexCoord2f(-1*si, 1*si);
     glVertex3f(-(si)*TileSize, 0, (si)*TileSize);

     glTexCoord2f(0, 1*si);
     glVertex3f(0, 0, (si)*TileSize);

   glEnd;


end;



end.
