(*** This unit handles drawing of TMesh in opengl.
It works by extending TMesh in TGlMesh,
which implements the abstract draw method.
Also, it always build a display list of the mesh if 
it doesn't have one already.
This unit replace the DrawMeshUnit (which has been discarded) *)
unit glmeshunit;

(*

This unit was written by Nicola {MSX} Lugato.

http://msx.rules.it
nicola@lugato.net
nicola.lugato@gmail.com

It is released under the term of
The GNU General Public License as published by the Free
Software Foundation.

*)
interface

uses meshunit, GL, SysUtils;

Type 

{** implements a OpenGl based Mesh. }
TGlMesh = class(TMesh)
  private
    {** the display list associated with this mesh }
    list:cardinal;                      
    {** builds up the display list with mesh data }
    procedure BuildDisplayList;         
    {** frees eventual display list. }
    procedure FreeDisplayList;           
    {** do the actual drawing of the object (without display list) }
    procedure DrawComplete;             
  public
    {** initialize the mesh }
    constructor Create;override;
    {** draw the mesh with the display list (eventually building one on the fly) }
	  procedure Draw; override;           
	  {** release all resources }
	  destructor Destroy; override;       
end;
 

implementation


constructor TGlMesh.Create;
begin
	inherited Create;
  list:=0;
end;

procedure TGlMesh.Draw;
begin
if list=0 then BuildDisplayList; // display list on the fly :P
glCallList(list);
end;

(*
  This draws the mesh directly without using display lists
 *)
procedure TGlMesh.DrawComplete;
var i:integer;
begin
 glBegin(GL_TRIANGLES);

for i:=0 to NumFace-1 do
 with Face[i] do
  begin

   glnormal3f(Vertex[point[1]].nx,Vertex[point[1]].ny,Vertex[point[1]].nz);
   glTexCoord2f(uv[1].u, uv[1].v);
   glVertex3f(Vertex[point[1]].x, Vertex[point[1]].y, Vertex[point[1]].z);

   glnormal3f(Vertex[point[2]].nx,Vertex[point[2]].ny,Vertex[point[2]].nz);
   glTexCoord2f(uv[2].u, uv[2].v);
   glVertex3f(Vertex[point[2]].x, Vertex[point[2]].y, Vertex[point[2]].z);

   glnormal3f(Vertex[point[3]].nx,Vertex[point[3]].ny,Vertex[point[3]].nz);
   glTexCoord2f(uv[3].u, uv[3].v);
   glVertex3f(Vertex[point[3]].x, Vertex[point[3]].y, Vertex[point[3]].z);
      
  end;
glEnd();
end;


procedure TGlMesh.BuildDisplayList;
begin
if list<>0 then exit;        // already created!
list:=glGenLists(1);         // obtain a new list
glNewList(list,GL_COMPILE);  // start drawing it!
DrawComplete;                // draw it!
glEndList;                   // finished drawing
end;


procedure TGlMesh.FreeDisplayList;
begin
if list=0 then exit;      // no list created
glDeleteLists(list, 1);
list:=0;
end;

destructor TGlMesh.Destroy;
begin
	freedisplaylist;
	inherited Destroy;
end;

end.
