{** This unit defines classes that wraps opengl textures.
A loading function to load textures from png is provided. }
unit textureunit;

(*

This unit was written by Nicola {MSX} Lugato.

http://msx.rules.it
nicola@lugato.net
nicola.lugato@gmail.com

It is released under the term of
The GNU General Public License as published by the Free
Software Foundation.


Description:

This unit defines classes that wraps opengl textures.
A loading function to load textures from png is provided.

Changelog:

2004-11-03: changelog started

*)


interface
uses
  Classes, SysUtils,
  GL, sdl, SDL_Image;


type
     TTexture = class
      name:String;
      texture:GLuInt;
      procedure LoadFrom(filename:string);
      procedure Bind;
     end;

     TTextureList=class(TList)
      function getTexture(index:integer):TTexture;
      property Texture[index:integer]:TTexture read getTexture; default;
      function getTextureByName(name:string):TTexture;
     end;


implementation

function LoadGLTexture(filename: string; var TextureID : GLuInt ): Boolean;
var
  // Create storage space for the texture
  TextureImage: PSDL_Surface;
begin
  // Load The Bitmap, Check For Errors, If Bitmap's Not Found Quit
  TextureImage := IMG_Load(pchar(filename));
  
  if ( TextureImage <> nil ) then
  begin
    result:=true;

    // Create Texture
    glGenTextures( 1, @TextureID );
    // Typical Texture Generation Using Data From The Bitmap
    glBindTexture( GL_TEXTURE_2D, TextureID );

    // Generate The Texture
    if TextureImage.format.bitsperpixel=24 then
      glTexImage2D( GL_TEXTURE_2D, 0, GL_RGB, TextureImage.w, TextureImage.h, 0, GL_RGB, GL_UNSIGNED_BYTE, TextureImage.pixels )
    else
      glTexImage2D( GL_TEXTURE_2D, 0, GL_RGBA, TextureImage.w, TextureImage.h, 0, GL_RGBA, GL_UNSIGNED_BYTE, TextureImage.pixels );

    // Linear Filtering
    // scale linearly when image bigger than texture
    glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR );
    // scale linearly when image smaller than texture
    glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR );
  end
  else
  begin
   raise exception.create('file '+filename+' not found');
   result:=false;
  end;

  // Free up any memory we may have used
  if ( TextureImage <> nil ) then
    SDL_FreeSurface( TextureImage );
end;


procedure TTexture.Bind;
begin
glBindTexture(GL_TEXTURE_2D, texture);
end;

procedure TTexture.LoadFrom(filename:string);
begin
LoadGLTexture(filename, texture);
end;

function TTextureList.getTexture(index:integer):TTexture;
begin
result:=TTexture(items[index]);
end;

function TTextureList.getTextureByName(name:string):TTexture;
var i:integer;
begin
 for i:=0 to count-1 do
  if texture[i].name=name then
  begin
   result:=texture[i];
   exit;
  end;
 result:=nil;
end;


end.
 
